﻿/**
* Sample code for the GetNonContractShipmentDetails Canada Post service.
* 
* The GetNonContractShipmentDetails service is used to retrieve XML details about the 
* shipping label, including system-generated values such as the tracking number 
* and final shipping point. 
*
* This sample is configured to access the Developer Program sandbox environment. 
* Use your development key username and password for the web service credentials.
* 
**/

using System;
using System.ServiceModel.Channels;
using System.ServiceModel;
using System.Configuration;
using System.Security.Authentication;
using System.Net;

namespace GetNCShipmentDetails
{
    class GetNCShipmentDetails
    {
        //TLS 1.2 support for .NET 3.5
        public const SslProtocols _Tls12 = (SslProtocols)0x00000C00;
        public const SecurityProtocolType Tls12 = (SecurityProtocolType)_Tls12;

        static void Main(string[] args)
        {
            ServicePointManager.SecurityProtocol = Tls12;

            // Your username, password and customer number are imported from the following file
            // CPCWS_SOAP_NCShipping_DotNet_Samples\SOAP\ncshipping\user.xml 
            var username = ConfigurationSettings.AppSettings["username"];
            var password = ConfigurationSettings.AppSettings["password"];
            var mailedBy = ConfigurationSettings.AppSettings["customerNumber"];

            NCShipmentPortTypeClient server = new NCShipmentPortTypeClient("NCShipmentPort", "https://ct.soa-gw.canadapost.ca/rs/soap/ncshipment/v4");
            server.ClientCredentials.UserName.UserName = username;
            server.ClientCredentials.UserName.Password = password;

            // Create Request
            getnoncontractshipmentdetailsrequest request = new getnoncontractshipmentdetailsrequest();
            request.locale = "FR";
            request.mailedby = mailedBy;
            request.shipmentid = "406951321983787352";

            // Disable timestamp on request
            BindingElementCollection elements = server.Endpoint.Binding.CreateBindingElements();
            elements.Find<SecurityBindingElement>().IncludeTimestamp = false;
            server.Endpoint.Binding = new CustomBinding(elements);

            String responseAsString = "";

            try
            {
                // Execute Request
                getnoncontractshipmentdetailsresponse response = server.GetNCShipmentDetails(request);

                // Retrieve values from response object
                if (response.Item.GetType() == typeof(NonContractShipmentDetailsType))
                {
                    NonContractShipmentDetailsType shipmentDetails = (NonContractShipmentDetailsType)response.Item;
                    responseAsString += "Tracking Pin: " + shipmentDetails.trackingpin + "\r\n";
                    responseAsString += "Service Code: " + shipmentDetails.deliveryspec.servicecode + "\r\n";
                    responseAsString += "Sender Postal Code: " + shipmentDetails.deliveryspec.sender.addressdetails.postalzipcode + "\r\n";
                    responseAsString += "Destination Postal Code: " + shipmentDetails.deliveryspec.destination.addressdetails.postalzipcode + "\r\n";
                }
                else
                {
                    messages msgs = (messages)response.Item;
                    foreach (var item in msgs.message)
                    {
                        responseAsString += "Error Code: " + item.code + "\r\n";
                        responseAsString += "Error Msg: " + item.description + "\r\n";
                    }
                }
            }
            catch (FaultException faultEx)
            {
                // SOAP Fault
                responseAsString += "Fault Code: " + faultEx.Code.Name + "\r\n";
                responseAsString += "Fault String: " + faultEx.Message + "\r\n";
            }
            catch (Exception ex)
            {
                // Misc Exception
                responseAsString += "ERROR: " + ex.Message + "\r\n";
            }

            Console.WriteLine(responseAsString);
            Console.WriteLine("Press any key to continue...");
            Console.ReadKey(true);

        }
    }
}
