﻿/**
* Sample code for the GetNCShipmentPublicKeyInfo Canada Post service.
* 
* The GetNCShipmentPublicKeyInfo service is used to retrieve XML details about the 
* shipping public key info. 
*
* This sample is configured to access the Developer Program sandbox environment. 
* Use your development key username and password for the web service credentials.
* 
**/

using System;
using System.ServiceModel.Channels;
using System.ServiceModel;
using System.Configuration;
using System.Security.Authentication;
using System.Net;
using System.IO;

namespace GetNCShipmentPublicKeyInfo
{
    class GetNCShipmentPublicKeyInfo
    {
        //TLS 1.2 support for .NET 3.5
        public const SslProtocols _Tls12 = (SslProtocols)0x00000C00;
        public const SecurityProtocolType Tls12 = (SecurityProtocolType)_Tls12;

        static void Main(string[] args)
        {
            ServicePointManager.SecurityProtocol = Tls12;

            // Your username, password and customer number are imported from the following file
            // CPCWS_SOAP_NCShipping_DotNet_Samples\SOAP\ncshipping\user.xml 
            var username = ConfigurationSettings.AppSettings["username"];
            var password = ConfigurationSettings.AppSettings["password"];
            var mailedBy = ConfigurationSettings.AppSettings["customerNumber"];

            NCShipmentPortTypeClient server = new NCShipmentPortTypeClient("NCShipmentPort", "https://ct.soa-gw.canadapost.ca/rs/soap/ncshipment/v4");
            server.ClientCredentials.UserName.UserName = username;
            server.ClientCredentials.UserName.Password = password;

            // Create Request
            getnoncontractshipmentpublickeyinforequest request = new getnoncontractshipmentpublickeyinforequest();
            request.locale = "FR";
            request.mailedby = mailedBy;
            request.shipmentid = "362741689788087765";

            // Disable timestamp on request
            BindingElementCollection elements = server.Endpoint.Binding.CreateBindingElements();
            elements.Find<SecurityBindingElement>().IncludeTimestamp = false;
            server.Endpoint.Binding = new CustomBinding(elements);

            String responseAsString = "";

            try
            {
                // Execute Request
                getnoncontractshipmentpublickeyinforesponse response = server.GetNCShipmentPublicKeyInfo(request);

                // Retrieve values from response object
                if (response.Item.GetType() == typeof(PublicKeyInfoType))
                {
                    PublicKeyInfoType publicKeyInfo = (PublicKeyInfoType)response.Item;
                    responseAsString += "Expiry Date: " + publicKeyInfo.expirydate+ "\r\n";
                    responseAsString += "Url: " + publicKeyInfo.url + "\r\n";
                    if (publicKeyInfo.qrcode != null)
                    {
                        responseAsString += "QR code Base64 string: " + Convert.ToBase64String(publicKeyInfo.qrcode) + "\r\n";
                        //write byte array to file
                        //File.WriteAllBytes("qr-code.jpeg", publicKeyInfo.qrcode);
                    }
                }
                else
                {
                    messages msgs = (messages)response.Item;
                    foreach (var item in msgs.message)
                    {
                        responseAsString += "Error Code: " + item.code + "\r\n";
                        responseAsString += "Error Msg: " + item.description + "\r\n";
                    }
                }
            }
            catch (FaultException faultEx)
            {
                // SOAP Fault
                responseAsString += "Fault Code: " + faultEx.Code.Name + "\r\n";
                responseAsString += "Fault String: " + faultEx.Message + "\r\n";
            }
            catch (Exception ex)
            {
                // Misc Exception
                responseAsString += "ERROR: " + ex.Message + "\r\n";
            }

            Console.WriteLine(responseAsString);
            Console.WriteLine("Press any key to continue...");
            Console.ReadKey(true);

        }
    }
}
