﻿/**
* Sample code for the GetNonContractShipmentReceipt Canada Post service.
* 
* The GetNonContractShipmentReceipt service is used to retrieve XML details 
* regarding the shipment paid for in the Create Non-Contract Shipment call. 
*
* This sample is configured to access the Developer Program sandbox environment. 
* Use your development key username and password for the web service credentials.
* 
**/

using System;
using System.ServiceModel.Channels;
using System.ServiceModel;
using System.Configuration;
using System.Security.Authentication;
using System.Net;

namespace GetNCShipmentReceipt
{
    class GetNCShipmentReceipt
    {
        //TLS 1.2 support for .NET 3.5
        public const SslProtocols _Tls12 = (SslProtocols)0x00000C00;
        public const SecurityProtocolType Tls12 = (SecurityProtocolType)_Tls12;

        static void Main(string[] args)
        {
            ServicePointManager.SecurityProtocol = Tls12;

            // Your username, password and customer number are imported from the following file
            // CPCWS_SOAP_NCShipping_DotNet_Samples\SOAP\ncshipping\user.xml 
            var username = ConfigurationSettings.AppSettings["username"];
            var password = ConfigurationSettings.AppSettings["password"];
            var mailedBy = ConfigurationSettings.AppSettings["customerNumber"];

            NCShipmentPortTypeClient server = new NCShipmentPortTypeClient("NCShipmentPort", "https://ct.soa-gw.canadapost.ca/rs/soap/ncshipment/v4");
            server.ClientCredentials.UserName.UserName = username;
            server.ClientCredentials.UserName.Password = password;

            // Create Request
            getnoncontractshipmentreceiptrequest request = new getnoncontractshipmentreceiptrequest();
            request.locale = "FR";
            request.mailedby = mailedBy;
            request.shipmentid = "406951321983787352";

            // Disable timestamp on request
            BindingElementCollection elements = server.Endpoint.Binding.CreateBindingElements();
            elements.Find<SecurityBindingElement>().IncludeTimestamp = false;
            server.Endpoint.Binding = new CustomBinding(elements);

            String responseAsString = "";

            try
            {
                // Execute Request
                getnoncontractshipmentreceiptresponse response = server.GetNCShipmentReceipt(request);

                // Retrieve values from response object
                if (response.Item.GetType() == typeof(NonContractShipmentReceiptType))
                {
                    NonContractShipmentReceiptType shipmentReceipt = (NonContractShipmentReceiptType)response.Item;
                    responseAsString += "Shipping Point Name: " + shipmentReceipt.shippingpointname + "\r\n";
                    responseAsString += "Service Code: " + shipmentReceipt.servicecode + "\r\n";
                    responseAsString += "Merchant Name: " + shipmentReceipt.ccreceiptdetails.merchantname + "\r\n";
                }
                else
                {
                    messages msgs = (messages)response.Item;
                    foreach (var item in msgs.message)
                    {
                        responseAsString += "Error Code: " + item.code + "\r\n";
                        responseAsString += "Error Msg: " + item.description + "\r\n";
                    }
                }
            }
            catch (FaultException faultEx)
            {
                // SOAP Fault
                responseAsString += "Fault Code: " + faultEx.Code.Name + "\r\n";
                responseAsString += "Fault String: " + faultEx.Message + "\r\n";
            }
            catch (Exception ex)
            {
                // Misc Exception
                responseAsString += "ERROR: " + ex.Message + "\r\n";
            }

            Console.WriteLine(responseAsString);
            Console.WriteLine("Press any key to continue...");
            Console.ReadKey(true);

        }
    }
}
