﻿/**
* Sample code for the RequestNCShipmentRefund Canada Post service.
* 
* The RequestNCShipmentRefund service is used to request a refund for 
* a shipment that has been transmitted. Making this call indicates 
* that the previously printed label is spoiled or will otherwise not be used.
* 
* Note that these are “requests for refund?, as refunding a shipment requires 
* proper verification before being actioned (i.e. ensure label has not been used, for example).
*
* This sample is configured to access the Developer Program sandbox environment. 
* Use your development key username and password for the web service credentials.
* 
**/

using System;
using System.ServiceModel.Channels;
using System.ServiceModel;
using System.Configuration;
using System.Security.Authentication;
using System.Net;

namespace RequestNCShipmentRefund
{
    class RequestNCShipmentRefund
    {
        //TLS 1.2 support for .NET 3.5
        public const SslProtocols _Tls12 = (SslProtocols)0x00000C00;
        public const SecurityProtocolType Tls12 = (SecurityProtocolType)_Tls12;

        static void Main(string[] args)
        {
            ServicePointManager.SecurityProtocol = Tls12;

            // Your username, password and customer number are imported from the following file
            // CPCWS_SOAP_NCShipping_DotNet_Samples\SOAP\ncshipping\user.xml 
            var username = ConfigurationSettings.AppSettings["username"];
            var password = ConfigurationSettings.AppSettings["password"];
            var mailedBy = ConfigurationSettings.AppSettings["customerNumber"];

            NCShipmentPortTypeClient server = new NCShipmentPortTypeClient("NCShipmentPort", "https://ct.soa-gw.canadapost.ca/rs/soap/ncshipment/v4");
            server.ClientCredentials.UserName.UserName = username;
            server.ClientCredentials.UserName.Password = password;

            // Create Request
            requestnoncontractshipmentrefundrequest request = new requestnoncontractshipmentrefundrequest();
            request.locale = "FR";
            request.mailedby = mailedBy;

            // Create shipment object to contain xml request
            request.shipmentid = "1234567890";
            NonContractShipmentRefundRequestType refundRequest = new NonContractShipmentRefundRequestType();
            refundRequest.email = "user@host.com";
            request.noncontractshipmentrefundrequest = refundRequest;

            // Disable timestamp on request
            BindingElementCollection elements = server.Endpoint.Binding.CreateBindingElements();
            elements.Find<SecurityBindingElement>().IncludeTimestamp = false;
            server.Endpoint.Binding = new CustomBinding(elements);

            String responseAsString = "";

            try
            {
                // Execute Request
                requestnoncontractshipmentrefundresponse response = server.RequestNCShipmentRefund(request);

                // Retrieve values from response object
                if (response.Item.GetType() == typeof(NonContractShipmentRefundRequestInfoType))
                {
                    NonContractShipmentRefundRequestInfoType shipmentRefundRequestInfo = (NonContractShipmentRefundRequestInfoType)response.Item;
                    responseAsString += "Service Ticket ID: " + shipmentRefundRequestInfo.serviceticketid+ "\r\n";
                    responseAsString += "Service Ticket Date: " + shipmentRefundRequestInfo.serviceticketdate + "\r\n";
                }
                else
                {
                    messages msgs = (messages)response.Item;
                    foreach (var item in msgs.message)
                    {
                        responseAsString += "Error Code: " + item.code + "\r\n";
                        responseAsString += "Error Msg: " + item.description + "\r\n";
                    }
                }
            }
            catch (FaultException faultEx)
            {
                // SOAP Fault
                responseAsString += "Fault Code: " + faultEx.Code.Name + "\r\n";
                responseAsString += "Fault String: " + faultEx.Message + "\r\n";
            }
            catch (Exception ex)
            {
                // Misc Exception
                responseAsString += "ERROR: " + ex.Message + "\r\n";
            }

            Console.WriteLine(responseAsString);
            Console.WriteLine("Press any key to continue...");
            Console.ReadKey(true);
        }
    }
}
