﻿/**
* Sample code for the GetMerchantRegistrationInfo Canada Post service.
* 
* The GetMerchantRegistrationInfo service is called by the ecommerce platform after 
* the merchant has completed the Canada Post sign-up process. This call returns
* merchant data such as customer number and merchant username and password. This
* information is necessary for the platform to perform web service shipping
* transactions for the merchant.
*
* This sample is configured to access the Developer Program sandbox environment. 
* Use your development key username and password for the web service credentials.
* 
**/

using System;
using System.ServiceModel.Channels;
using System.ServiceModel;
using System.Configuration;
using System.Security.Authentication;
using System.Net;

namespace GetMerchantRegistrationInfo
{
    class GetMerchantRegistrationInfo
    {
        //TLS 1.2 support for .NET 3.5
        public const SslProtocols _Tls12 = (SslProtocols)0x00000C00;
        public const SecurityProtocolType Tls12 = (SecurityProtocolType)_Tls12;

        static void Main(string[] args)
        {
            ServicePointManager.SecurityProtocol = Tls12;

            // Your username and password are imported from the following file
            // CPCWS_SOAP_Platforms_DotNet_Samples\SOAP\platforms\user.xml 
            var username = ConfigurationSettings.AppSettings["username"];
            var password = ConfigurationSettings.AppSettings["password"];

            MerchantRegistrationPortTypeClient server = new MerchantRegistrationPortTypeClient("MerchantRegistrationPort", "https://ct.soa-gw.canadapost.ca/ot/soap/merchant/registration");
            server.ClientCredentials.UserName.UserName = username;
            server.ClientCredentials.UserName.Password = password;

            // Create Request
            getmerchantregistrationinforequest request = new getmerchantregistrationinforequest();
            request.locale = "FR";
            request.tokenid = "1111111111111111111111";

            // Disable timestamp on request
            BindingElementCollection elements = server.Endpoint.Binding.CreateBindingElements();
            elements.Find<SecurityBindingElement>().IncludeTimestamp = false;
            server.Endpoint.Binding = new CustomBinding(elements);

            String responseAsString = "";

            try
            {
                // Execute Request
                getmerchantregistrationinforesponse response = server.GetMerchantRegistrationInfo(request);

                // Retrieve values from response object
                if (response.Item.GetType() == typeof(getmerchantregistrationinforesponseMerchantinfo))
                {
                    // Retrieve values from response object
                    getmerchantregistrationinforesponseMerchantinfo merchantInfo = (getmerchantregistrationinforesponseMerchantinfo)response.Item;
                    if (merchantInfo.customernumber != null)
                    {
                        responseAsString += "Customer Number: " + merchantInfo.customernumber + "\r\n";
                    }
                    if (merchantInfo.contractnumber != null)
                    {
                        responseAsString += "Contract Number: " + merchantInfo.contractnumber + "\r\n";
                    }
                    if (merchantInfo.merchantusername != null)
                    {
                        responseAsString += "Merchant Username: " + merchantInfo.merchantusername + "\r\n";
                    }
                    if (merchantInfo.merchantpassword != null)
                    {
                        responseAsString += "Merchant Password: " + merchantInfo.merchantpassword + "\r\n";
                    }
                    responseAsString += "Has Default CC: " + merchantInfo.hasdefaultcreditcard + "\r\n";
                }
                else
                {
                    MessagesType msgs = (MessagesType)response.Item;
                    foreach (var item in msgs.message)
                    {
                        responseAsString += "Error Code: " + item.code + "\r\n";
                        responseAsString += "Error Msg: " + item.description + "\r\n";
                    }
                }
            }
            catch (FaultException faultEx)
            {
                // SOAP Fault
                responseAsString += "Fault Code: " + faultEx.Code.Name + "\r\n";
                responseAsString += "Fault String: " + faultEx.Message + "\r\n";
            }
            catch (Exception ex)
            {
                // Misc Exception
                responseAsString += "ERROR: " + ex.Message + "\r\n";
            }

            Console.WriteLine(responseAsString);
            Console.WriteLine("Press any key to continue...");
            Console.ReadKey(true);

        }
    }
}
