﻿/**
* Sample code for the GetPostOfficeDetail Canada Post service.
* 
* The GetPostOfficeDetail service returns Post Office details such as the daily 
* hours of operation for a given Post Office as determined by the first call.
*
* This sample is configured to access the Developer Program sandbox environment. 
* Use your development key username and password for the web service credentials.
* 
**/

using System;
using System.ServiceModel.Channels;
using System.ServiceModel;
using System.Configuration;
using System.Security.Authentication;
using System.Net;

namespace GetPostOfficeDetail
{
    class GetPostOfficeDetail
    {
        //TLS 1.2 support for .NET 3.5
        public const SslProtocols _Tls12 = (SslProtocols)0x00000C00;
        public const SecurityProtocolType Tls12 = (SecurityProtocolType)_Tls12;

        static void Main(string[] args)
        {
            ServicePointManager.SecurityProtocol = Tls12;

            // Your username and password are imported from the following file
            // CPCWS_SOAP_PostOffice_DotNet_Samples\SOAP\postoffice\user.xml 
            var username = ConfigurationSettings.AppSettings["username"];
            var password = ConfigurationSettings.AppSettings["password"];

            PostOfficePortTypeClient server = new PostOfficePortTypeClient("PostOfficePort", "https://ct.soa-gw.canadapost.ca/rs/soap/postoffice");
            server.ClientCredentials.UserName.UserName = username;
            server.ClientCredentials.UserName.Password = password;

            // Create Request
            getpostofficedetailrequest request = new getpostofficedetailrequest();
            request.locale = "FR";
            request.officeid = "102978";

            // Disable timestamp on request
            BindingElementCollection elements = server.Endpoint.Binding.CreateBindingElements();
            elements.Find<SecurityBindingElement>().IncludeTimestamp = false;
            server.Endpoint.Binding = new CustomBinding(elements);

            String responseAsString = "";

            try
            {
                // Execute Request
                getpostofficedetailresponse response = server.GetPostOfficeDetail(request);

                // Retrieve values from response object
                if (response.Item.GetType() == typeof(getpostofficedetailresponsePostofficedetail))
                {
                    // Retrieve values from response object
                    getpostofficedetailresponsePostofficedetail postOffice = (getpostofficedetailresponsePostofficedetail)response.Item;
                    responseAsString += "Office Name: " + postOffice.name + "\r\n";
                    responseAsString += "Address: " + postOffice.address.officeaddress + "\r\n\r\n";
                    foreach (var hoursListItem in postOffice.hourslist)
                    {
                        responseAsString += "Weekday: " + hoursListItem.day.ToString().Substring(4) +"\r\n";
                        foreach (var timeItem in hoursListItem.time)
                        {
                            responseAsString += "Time: " + timeItem + "\r\n";
                        }
                        responseAsString += "\r\n";
                    }  
                }
                else
                {
                    messages msgs = (messages)response.Item;
                    foreach (var item in msgs.message)
                    {
                        responseAsString += "Error Code: " + item.code + "\r\n";
                        responseAsString += "Error Msg: " + item.description + "\r\n";
                    }
                }
            }
            catch (FaultException faultEx)
            {
                // SOAP Fault
                responseAsString += "Fault Code: " + faultEx.Code.Name + "\r\n";
                responseAsString += "Fault String: " + faultEx.Message + "\r\n";
            }
            catch (Exception ex)
            {
                // Misc Exception
                responseAsString += "ERROR: " + ex.Message + "\r\n";
            }

            Console.WriteLine(responseAsString);
            Console.WriteLine("Press any key to continue...");
            Console.ReadKey(true);

        }
    }
}
