﻿/**
* Sample code for the GetOptions Canada Post service.
* 
* The GetOptions service  returns information about a given add-on option such 
* as how it is used and whether it requires or conflicts with other options. 
*
* This sample is configured to access the Developer Program sandbox environment. 
* Use your development key username and password for the web service credentials.
* 
**/

using System;
using System.ServiceModel.Channels;
using System.ServiceModel;
using System.Configuration;
using System.Security.Authentication;
using System.Net;

namespace GetOption
{
    class GetOption
    {
        //TLS 1.2 support for .NET 3.5
        public const SslProtocols _Tls12 = (SslProtocols)0x00000C00;
        public const SecurityProtocolType Tls12 = (SecurityProtocolType)_Tls12;

        static void Main(string[] args)
        {
            ServicePointManager.SecurityProtocol = Tls12;

            // Your username and password are imported from the following file
            // CPCWS_SOAP_Rating_DotNet_Samples\SOAP\rating\user.xml 
            var username = ConfigurationSettings.AppSettings["username"];
            var password = ConfigurationSettings.AppSettings["password"];

            RatingPortTypeClient server = new RatingPortTypeClient("RatingPort", "https://ct.soa-gw.canadapost.ca/rs/soap/rating/v4");
            server.ClientCredentials.UserName.UserName = username;
            server.ClientCredentials.UserName.Password = password;

            // Create Request
            getoptionrequest request = new getoptionrequest();
            request.locale = "FR";
            request.optioncode = "DC";

            // Disable timestamp on request
            BindingElementCollection elements = server.Endpoint.Binding.CreateBindingElements();
            elements.Find<SecurityBindingElement>().IncludeTimestamp = false;
            server.Endpoint.Binding = new CustomBinding(elements);

            String responseAsString = "";

            try
            {
                // Execute Request
                getoptionresponse response = server.GetOption(request);

                // Retrieve values from response object
                if (response.Item.GetType() == typeof(getoptionresponseOption))
                {
                    getoptionresponseOption option = (getoptionresponseOption)response.Item;
                    responseAsString += "Option Name: " + option.optionname + "\r\n";
                    responseAsString += "Option Code: " + option.optioncode + "\r\n";
                    responseAsString += "Option Class: " + option.optionclass + "\r\n";
                    responseAsString += "Print on Label: " + option.printsonlabel + "\r\n";
                    responseAsString += "Qualifier Required: " + option.qualifierrequired + "\r\n";
                }
                else
                {
                    messages msgs = (messages)response.Item;
                    foreach (var item in msgs.message)
                    {
                        responseAsString += "Error Code: " + item.code + "\r\n";
                        responseAsString += "Error Msg: " + item.description + "\r\n";
                    }
                }
            }
            catch (FaultException faultEx)
            {
                // SOAP Fault
                responseAsString += "Fault Code: " + faultEx.Code.Name + "\r\n";
                responseAsString += "Fault String: " + faultEx.Message + "\r\n";
            }
            catch (Exception ex)
            {
                // Misc Exception
                responseAsString += "ERROR: " + ex.Message + "\r\n";
            }

            Console.WriteLine(responseAsString);
            Console.WriteLine("Press any key to continue...");
            Console.ReadKey(true);

        }
    }
}
