﻿/**
* Sample code for the GetService Canada Post service.
* 
* The GetService service  returns details of a given postal service in 
* terms of the min/max weight and dimensions offered by the postal service. 
* Also returned are details about the available add-on options. 
*
* This sample is configured to access the Developer Program sandbox environment. 
* Use your development key username and password for the web service credentials.
* 
**/

using System;
using System.ServiceModel.Channels;
using System.ServiceModel;
using System.Configuration;
using System.Security.Authentication;
using System.Net;

namespace GetService
{
    class GetService
    {
        //TLS 1.2 support for .NET 3.5
        public const SslProtocols _Tls12 = (SslProtocols)0x00000C00;
        public const SecurityProtocolType Tls12 = (SecurityProtocolType)_Tls12;

        static void Main(string[] args)
        {
            ServicePointManager.SecurityProtocol = Tls12;

            // Your username and password are imported from the following file
            // CPCWS_SOAP_Rating_DotNet_Samples\SOAP\rating\user.xml 
            var username = ConfigurationSettings.AppSettings["username"];
            var password = ConfigurationSettings.AppSettings["password"];

            RatingPortTypeClient server = new RatingPortTypeClient("RatingPort", "https://ct.soa-gw.canadapost.ca/rs/soap/rating/v4");
            server.ClientCredentials.UserName.UserName = username;
            server.ClientCredentials.UserName.Password = password;

            // Create Request
            getservicerequest request = new getservicerequest();
            request.locale = "FR";
            request.servicecode = "DOM.EP";
            request.destinationcountrycode = "CA";

            // Disable timestamp on request
            BindingElementCollection elements = server.Endpoint.Binding.CreateBindingElements();
            elements.Find<SecurityBindingElement>().IncludeTimestamp = false;
            server.Endpoint.Binding = new CustomBinding(elements);

            String responseAsString = "";

            try
            {
                // Execute Request
                getserviceresponse response = server.GetService(request);

                // Retrieve values from response object
                if (response.Item.GetType() == typeof(getserviceresponseService))
                {
                    getserviceresponseService service = (getserviceresponseService)response.Item;
                    responseAsString += "Service Name: " + service.servicename + "\r\n";
                    responseAsString += "Service Code: " + service.servicecode + "\r\n\r\n";
                    responseAsString += "Options:\r\n";
                    foreach (var option in service.options)
                    {
                        responseAsString += "Option Name: " + option.optionname + "\r\n";
                        responseAsString += "Option Code: " + option.optioncode + "\r\n\r\n";
                    }
                }
                else
                {
                    messages msgs = (messages)response.Item;
                    foreach (var item in msgs.message)
                    {
                        responseAsString += "Error Code: " + item.code + "\r\n";
                        responseAsString += "Error Msg: " + item.description + "\r\n";
                    }
                }
            }
            catch (FaultException faultEx)
            {
                // SOAP Fault
                responseAsString += "Fault Code: " + faultEx.Code.Name + "\r\n";
                responseAsString += "Fault String: " + faultEx.Message + "\r\n";
            }
            catch (Exception ex)
            {
                // Misc Exception
                responseAsString += "ERROR: " + ex.Message + "\r\n";
            }

            Console.WriteLine(responseAsString);
            Console.WriteLine("Press any key to continue...");
            Console.ReadKey(true);

        }
    }
}
