﻿/**
* Sample code for the CreateAuthorizedReturn Canada Post service.
* 
* The CreateAuthorizedReturn service is used to create a return shipping label 
* when the sender address and approximate package weight are known. This type of 
* label is typically emailed through a self-serve or service agent process to the
* individual returning the item.
*
* This sample is configured to access the Developer Program sandbox environment. 
* Use your development key username and password for the web service credentials.
* 
**/

using System;
using System.ServiceModel.Channels;
using System.ServiceModel;
using System.Configuration;
using System.Security.Authentication;
using System.Net;

namespace CreateAuthReturn
{
    class CreateAuthReturn
    {
        //TLS 1.2 support for .NET 3.5
        public const SslProtocols _Tls12 = (SslProtocols)0x00000C00;
        public const SecurityProtocolType Tls12 = (SecurityProtocolType)_Tls12;

        static void Main(string[] args)
        {
            ServicePointManager.SecurityProtocol = Tls12;

            // Your username, password and customer number are imported from the following file
            // CPCWS_SOAP_Returns_DotNet_Samples\SOAP\returns\user.xml 
            var username = ConfigurationSettings.AppSettings["username"];
            var password = ConfigurationSettings.AppSettings["password"];
            var mailedBy = ConfigurationSettings.AppSettings["customerNumber"];

            AuthReturnPortTypeClient server = new AuthReturnPortTypeClient("AuthReturnPort", "https://ct.soa-gw.canadapost.ca/rs/soap/authreturn/v2");
            server.ClientCredentials.UserName.UserName = username;
            server.ClientCredentials.UserName.Password = password;

            // Create Request
            createauthorizedreturnrequest request = new createauthorizedreturnrequest();
            request.locale = "FR";
            request.mailedby = mailedBy;

            // Create AuthorizedReturnType object
            request.authorizedreturn = new AuthorizedReturnType();
            AuthorizedReturnType authReturn = new AuthorizedReturnType();
            request.authorizedreturn = authReturn;
            authReturn.returner = new ReturnerType();
            authReturn.returner.domesticaddress = new DomesticAddressDetailsType();
            authReturn.receiver = new ReceiverType();
            authReturn.receiver.domesticaddress = new DomesticAddressDetailsType();
            authReturn.parcelcharacteristics = new ParcelCharacteristicsType();
            authReturn.parcelcharacteristics.dimensions = new ParcelCharacteristicsTypeDimensions();
            authReturn.printpreferences = new PrintPreferencesType();
            authReturn.settlementinfo = new AuthSettlementInfoType();

            // Populate AuthorizedReturnType object
            authReturn.returner.domesticaddress.addressline1 = "502 MAIN ST N";
            authReturn.returner.domesticaddress.city = "MONTREAL";
            authReturn.returner.domesticaddress.province = "QC";
            authReturn.returner.domesticaddress.postalcode = "H2B1A0";
            authReturn.returner.name = "Bulma";
            authReturn.returner.company = "Capsule Corp.";
            authReturn.receiver.domesticaddress.addressline1 = "123 Postal Drive";
            authReturn.receiver.domesticaddress.city = "Ottawa";
            authReturn.receiver.domesticaddress.province = "ON";
            authReturn.receiver.domesticaddress.postalcode = "K1P5Z9";
            authReturn.receiver.name = "John Doe";
            authReturn.receiver.company = "ACME Corp.";
            authReturn.parcelcharacteristics.dimensions.height = 6;
            authReturn.parcelcharacteristics.dimensions.length = 12;
            authReturn.parcelcharacteristics.dimensions.width = 9;
            authReturn.parcelcharacteristics.weight = 5;
            authReturn.printpreferences.encoding = PrintPreferencesTypeEncoding.PDF;
            authReturn.printpreferences.outputformat = PrintPreferencesTypeOutputformat.Item85x11;
            authReturn.settlementinfo.contractid = "0042708517";
            authReturn.servicecode = "DOM.EP";

            // Disable timestamp on request
            BindingElementCollection elements = server.Endpoint.Binding.CreateBindingElements();
            elements.Find<SecurityBindingElement>().IncludeTimestamp = false;
            server.Endpoint.Binding = new CustomBinding(elements);

            String responseAsString = "";

            try
            {
                // Execute Request
                createauthorizedreturnresponse response = server.CreateAuthorizedReturn(request);

                // Retrieve values from response object
                if (response.Item.GetType() == typeof(AuthorizedReturnInfoType))
                {
                    AuthorizedReturnInfoType authReturnInfo = (AuthorizedReturnInfoType)response.Item;
                    responseAsString += "Tracking Pin: " + authReturnInfo.trackingpin + "\r\n";
                    foreach (var artifact in authReturnInfo.artifacts)
                    {
                        responseAsString += "Artifact Id:  " + artifact.artifactid + "\r\n";
                    }
                }
                else
                {
                    messages msgs = (messages)response.Item;
                    foreach (var item in msgs.message)
                    {
                        responseAsString += "Error Code: " + item.code + "\r\n";
                        responseAsString += "Error Msg: " + item.description + "\r\n";
                    }
                }
            }
            catch (FaultException faultEx)
            {
                // SOAP Fault
                responseAsString += "Fault Code: " + faultEx.Code.Name + "\r\n";
                responseAsString += "Fault String: " + faultEx.Message + "\r\n";
            }
            catch (Exception ex)
            {
                // Misc Exception
                responseAsString += "ERROR: " + ex.Message + "\r\n";
            }

            Console.WriteLine(responseAsString);
            Console.WriteLine("Press any key to continue...");
            Console.ReadKey(true);

        }
    }
}
