﻿/**
* Sample code for the CreateOpenReturnTemplate Canada Post service.
* 
* The CreateOpenReturnTemplate service is used to request creation of generic
* labels for retrieval and printing. The sender address and parcel weight are
* unknown. Each label has a unique barcode, but the rest of the data is the same.
* The labels can be distributed as part of the original shipment, or sent to a
* specific individual. 
*
* This sample is configured to access the Developer Program sandbox environment. 
* Use your development key username and password for the web service credentials.
* 
**/

using System;
using System.ServiceModel.Channels;
using System.ServiceModel;
using System.Configuration;
using System.Security.Authentication;
using System.Net;

namespace CreateOpenReturnTemplate
{
    class CreateOpenReturnTemplate
    {
        //TLS 1.2 support for .NET 3.5
        public const SslProtocols _Tls12 = (SslProtocols)0x00000C00;
        public const SecurityProtocolType Tls12 = (SecurityProtocolType)_Tls12;

        static void Main(string[] args)
        {
            ServicePointManager.SecurityProtocol = Tls12;

            // Your username, password and customer number are imported from the following file
            // CPCWS_SOAP_Returns_DotNet_Samples\SOAP\returns\user.xml 
            var username = ConfigurationSettings.AppSettings["username"];
            var password = ConfigurationSettings.AppSettings["password"];
            var mailedBy = ConfigurationSettings.AppSettings["customerNumber"];

            OpenReturnPortTypeClient server = new OpenReturnPortTypeClient("OpenReturnPort", "https://ct.soa-gw.canadapost.ca/rs/soap/openreturn/v2");
            server.ClientCredentials.UserName.UserName = username;
            server.ClientCredentials.UserName.Password = password;

            // Create Request
            createopenreturntemplaterequest request = new createopenreturntemplaterequest();
            request.locale = "FR";
            request.mailedby = mailedBy;

            // Create OpenReturnType object
            request.openreturn = new OpenReturnType();
            OpenReturnType openReturn = new OpenReturnType();
            request.openreturn = openReturn;
            openReturn.receiver = new DomesticAddressType();
            openReturn.receiver.domesticaddress = new DomesticAddressDetailsType();
            openReturn.printpreferences = new PrintPreferencesType();
            openReturn.settlementinfo = new SettlementInfoType();

            // Populate OpenReturnType object
            openReturn.maxnumberofartifacts = "10";
            openReturn.receiver.domesticaddress.addressline1 = "123 Postal Drive";
            openReturn.receiver.domesticaddress.city = "Ottawa";
            openReturn.receiver.domesticaddress.province = "ON";
            openReturn.receiver.domesticaddress.postalcode = "K1P5Z9";
            openReturn.receiver.name = "John Doe";
            openReturn.receiver.company = "ACME Corp.";
            openReturn.printpreferences.encoding = PrintPreferencesTypeEncoding.PDF;
            openReturn.printpreferences.outputformat = PrintPreferencesTypeOutputformat.Item85x11;
            openReturn.settlementinfo.contractid = "0042708517";
            openReturn.servicecode = "DOM.EP";

            // Disable timestamp on request
            BindingElementCollection elements = server.Endpoint.Binding.CreateBindingElements();
            elements.Find<SecurityBindingElement>().IncludeTimestamp = false;
            server.Endpoint.Binding = new CustomBinding(elements);

            String responseAsString = "";

            try
            {
                // Execute Request
                createopenreturntemplateresponse response = server.CreateOpenReturnTemplate(request);

                // Retrieve values from response object
                if (response.Item.GetType() == typeof(OpenReturnInfoType))
                {
                    OpenReturnInfoType openReturnInfo = (OpenReturnInfoType)response.Item;
                    responseAsString += "Artifacts Remaining: " + openReturnInfo.artifactsremaining + "\r\n";
                    responseAsString += "Template Id: " + openReturnInfo.templateid + "\r\n";
                }
                else
                {
                    messages msgs = (messages)response.Item;
                    foreach (var item in msgs.message)
                    {
                        responseAsString += "Error Code: " + item.code + "\r\n";
                        responseAsString += "Error Msg: " + item.description + "\r\n";
                    }
                }
            }
            catch (FaultException faultEx)
            {
                // SOAP Fault
                responseAsString += "Fault Code: " + faultEx.Code.Name + "\r\n";
                responseAsString += "Fault String: " + faultEx.Message + "\r\n";
            }
            catch (Exception ex)
            {
                // Misc Exception
                responseAsString += "ERROR: " + ex.Message + "\r\n";
            }

            Console.WriteLine(responseAsString);
            Console.WriteLine("Press any key to continue...");
            Console.ReadKey(true);

        }
    }
}
