﻿/**
* Sample code for the GetManifestArtifactId Canada Post service.
* 
* The GetManifestArtifactId service is used to retrieve the artifact-id number of the
* artifact (label) for a given manifest. 
*
* This sample is configured to access the Developer Program sandbox environment. 
* Use your development key username and password for the web service credentials.
* 
**/

using System;
using System.ServiceModel.Channels;
using System.ServiceModel;
using System.Configuration;
using System.Security.Authentication;
using System.Net;

namespace GetManifestArtifactId
{
    class GetManifestArtifactId
    {
        //TLS 1.2 support for .NET 3.5
        public const SslProtocols _Tls12 = (SslProtocols)0x00000C00;
        public const SecurityProtocolType Tls12 = (SecurityProtocolType)_Tls12;

        static void Main(string[] args)
        {
            ServicePointManager.SecurityProtocol = Tls12;

            // Your username, password and customer number are imported from the following file
            // CPCWS_SOAP_Shipping_DotNet_Samples\SOAP\shipping\user.xml 
            var username = ConfigurationSettings.AppSettings["username"];
            var password = ConfigurationSettings.AppSettings["password"];
            var mailedBy = ConfigurationSettings.AppSettings["customerNumber"];

            ManifestPortTypeClient server = new ManifestPortTypeClient("ManifestPort", "https://ct.soa-gw.canadapost.ca/rs/soap/manifest/v8");
            server.ClientCredentials.UserName.UserName = username;
            server.ClientCredentials.UserName.Password = password;

            // Create Request
            getmanifestartifactidrequest request = new getmanifestartifactidrequest();
            request.locale = "FR";
            request.mailedby = mailedBy;
            request.manifestid = "347891314723499921";

            // Disable timestamp on request
            BindingElementCollection elements = server.Endpoint.Binding.CreateBindingElements();
            elements.Find<SecurityBindingElement>().IncludeTimestamp = false;
            server.Endpoint.Binding = new CustomBinding(elements);

            String responseAsString = "";

            try
            {
                // Execute Request
                getmanifestartifactidresponse response = server.GetManifestArtifactId(request);

                // Retrieve values from response object
                if (response.Item.GetType() == typeof(ManifestType))
                {
                    ManifestType manifest = (ManifestType)response.Item;
                    responseAsString += "PO Number: " + manifest.ponumber + "\r\n";
                    responseAsString += "Artifact Id: " + manifest.artifactid + "\r\n";
                }
                else
                {
                    messages msgs = (messages)response.Item;
                    foreach (var item in msgs.message)
                    {
                        responseAsString += "Error Code: " + item.code + "\r\n";
                        responseAsString += "Error Msg: " + item.description + "\r\n";
                    }
                }
            }
            catch (FaultException faultEx)
            {
                // SOAP Fault
                responseAsString += "Fault Code: " + faultEx.Code.Name + "\r\n";
                responseAsString += "Fault String: " + faultEx.Message + "\r\n";
            }
            catch (Exception ex)
            {
                // Misc Exception
                responseAsString += "ERROR: " + ex.Message + "\r\n";
            }

            Console.WriteLine(responseAsString);
            Console.WriteLine("Press any key to continue...");
            Console.ReadKey(true);

        }
    }
}
