﻿/**
* Sample code for the GetShipments Canada Post service.
* 
* The GetShipments service is used to retrieve the shipment IDs for a collection
* of shipments not yet transmitted (members of a group), or to retrieve the
* shipment IDs for a collection of shipments that have been transmitted (members
* of a manifest).
*
* This sample is configured to access the Developer Program sandbox environment. 
* Use your development key username and password for the web service credentials.
* 
**/

using System;
using System.ServiceModel.Channels;
using System.ServiceModel;
using System.Configuration;
using System.Security.Authentication;
using System.Net;

namespace GetShipments
{
    class GetShipments
    {
        //TLS 1.2 support for .NET 3.5
        public const SslProtocols _Tls12 = (SslProtocols)0x00000C00;
        public const SecurityProtocolType Tls12 = (SecurityProtocolType)_Tls12;

        static void Main(string[] args)
        {
            ServicePointManager.SecurityProtocol = Tls12;

            // Your username, password and customer number are imported from the following file
            // CPCWS_SOAP_Shipping_DotNet_Samples\SOAP\shipping\user.xml 
            var username = ConfigurationSettings.AppSettings["username"];
            var password = ConfigurationSettings.AppSettings["password"];
            var mailedBy = ConfigurationSettings.AppSettings["customerNumber"];

            ShipmentPortTypeClient server = new ShipmentPortTypeClient("ShipmentPort", "https://ct.soa-gw.canadapost.ca/rs/soap/shipment/v8");
            server.ClientCredentials.UserName.UserName = username;
            server.ClientCredentials.UserName.Password = password;
            
            // Create Request
            getshipmentsrequest request = new getshipmentsrequest();
            request.locale = "FR";
            request.mailedby = mailedBy;

            request.shipmentsearchkey = new getshipmentsrequestShipmentsearchkey();
            
            // GET group Shipments

            regularshippingparameters regularShippingParameters = new regularshippingparameters();
            regularShippingParameters.ItemElementName = ItemChoiceType.groupid;
            regularShippingParameters.Item = "123456";
            request.shipmentsearchkey.ItemElementName = ItemChoiceType1.regularshippingparameters;
            request.shipmentsearchkey.Item = regularShippingParameters;
            

            
            // GET manifest Shipments
            /*
            regularshippingparameters regularShippingRequest = new regularshippingparameters();
            regularShippingRequest.ItemElementName = ItemChoiceType.manifestid;
            regularShippingRequest.Item = "347891314723499921";
            request.shipmentsearchkey.ItemElementName = ItemChoiceType1.regularshippingparameters;
            request.shipmentsearchkey.Item = regularShippingRequest;
            */


            // GET no-manifest Shipments
            /*
            nomanifestshippingparameters noManifestRequest = new nomanifestshippingparameters();
            noManifestRequest.noManifest = true;
            noManifestRequest.date = new DateTime(2013, 09, 10);
            noManifestRequest.dateSpecified = true;
            noManifestRequest.limit = "50";
            request.shipmentsearchkey.ItemElementName = ItemChoiceType1.nomanifestshippingparameters;
            request.shipmentsearchkey.Item = noManifestRequest;
            */


            // Disable timestamp on request
            BindingElementCollection elements = server.Endpoint.Binding.CreateBindingElements();
            elements.Find<SecurityBindingElement>().IncludeTimestamp = false;
            server.Endpoint.Binding = new CustomBinding(elements);

            String responseAsString = "";

            try
            {
                // Execute Request
                getshipmentsresponse response = server.GetShipments(request);
                
                // Retrieve values from response object
                if (response.Item.GetType() == typeof(ShipmentsType))
                {
                    ShipmentsType shipments = (ShipmentsType)response.Item;
                    if (shipments.Items != null)
                    {
                        foreach (var shipmentId in shipments.Items)
                        {
                            responseAsString += "Shipment Id:  " + shipmentId + "\r\n";
                        }   
                    }
                    else
                    {
                        responseAsString += "No shipments returned.\r\n";
                    }
                }
                else
                {
                    messages msgs = (messages)response.Item;
                    foreach (var item in msgs.message)
                    {
                        responseAsString += "Error Code: " + item.code + "\r\n";
                        responseAsString += "Error Msg: " + item.description + "\r\n";
                    }
                }
            }
            catch (FaultException faultEx)
            {
                // SOAP Fault
                responseAsString += "Fault Code: " + faultEx.Code.Name + "\r\n";
                responseAsString += "Fault String: " + faultEx.Message + "\r\n";
            }
            catch (Exception ex)
            {
                // Misc Exception
                responseAsString += "ERROR: " + ex.Message + "\r\n";
            }

            Console.WriteLine(responseAsString);
            Console.WriteLine("Press any key to continue...");
            Console.ReadKey(true);

        }
    }
}
